/* BdfServer - Copyright (c) 2015-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.bdfserver.json;

import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;
import net.fichotheque.SubsetKey;
import net.fichotheque.thesaurus.Motcle;
import net.fichotheque.thesaurus.Thesaurus;
import net.fichotheque.thesaurus.metadata.ThesaurusMetadata;
import net.fichotheque.utils.CroisementUtils;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.collation.map.SortedCollatedKeyMap;


/**
 *
 * @author Vincent Calame
 */
public final class ThesaurusSuggestionJson {

    private ThesaurusSuggestionJson() {

    }

    public static String toString(Thesaurus thesaurus, Lang thesaurusLang, String idalphaSeparator, SubsetKey destinationSubsetKey) {
        short type = thesaurus.getThesaurusMetadata().getThesaurusType();
        switch (type) {
            case ThesaurusMetadata.IDALPHA_TYPE:
                return idalphaToString(thesaurus, thesaurusLang, idalphaSeparator, destinationSubsetKey);
            case ThesaurusMetadata.MULTI_TYPE:
                return multiToString(thesaurus, thesaurusLang, destinationSubsetKey);
            case ThesaurusMetadata.BABELIEN_TYPE:
                return babelienToString(thesaurus, thesaurusLang, destinationSubsetKey);
            default:
                throw new SwitchException("Unknonw thesaurus type: " + type);
        }

    }

    private static String multiToString(Thesaurus thesaurus, Lang thesaurusLang, SubsetKey destinationSubsetKey) {
        SortedCollatedKeyMap<Label> collatedKeyMap = new SortedCollatedKeyMap<Label>(thesaurusLang.toLocale());
        for (Motcle motcle : thesaurus.getMotcleList()) {
            if (CroisementUtils.isShouldNotCroisement(motcle, destinationSubsetKey)) {
                continue;
            }
            Label label = motcle.getLabels().getLangPartCheckedLabel(thesaurusLang);
            if (label != null) {
                collatedKeyMap.putValue(label.getLabelString(), label);
            }
        }
        StringBuilder buf = new StringBuilder();
        buf.append("[");
        boolean first = true;
        for (Label label : collatedKeyMap.values()) {
            if (first) {
                first = false;
            } else {
                buf.append(",");
            }
            buf.append("{");
            buf.append("\"value\":\"");
            escape(buf, label.getLabelString());
            buf.append("\"");
            buf.append("}");
        }
        buf.append("]");
        return buf.toString();
    }

    private static String babelienToString(Thesaurus thesaurus, Lang thesaurusLang, SubsetKey destinationSubsetKey) {
        SortedCollatedKeyMap<Label> collatedKeyMap = new SortedCollatedKeyMap<Label>(thesaurusLang.toLocale());
        for (Motcle motcle : thesaurus.getMotcleList()) {
            if (CroisementUtils.isShouldNotCroisement(motcle, destinationSubsetKey)) {
                continue;
            }
            Label label = motcle.getBabelienLabel();
            collatedKeyMap.putValue(label.getLabelString(), label);
        }
        StringBuilder buf = new StringBuilder();
        buf.append("[");
        boolean first = true;
        for (Label label : collatedKeyMap.values()) {
            if (first) {
                first = false;
            } else {
                buf.append(",");
            }
            buf.append("{");
            buf.append("\"value\":\"");
            escape(buf, label.getLabelString());
            buf.append("\"");
            buf.append("}");
        }
        buf.append("]");
        return buf.toString();
    }

    private static String idalphaToString(Thesaurus thesaurus, Lang thesaurusLang, String idalphaSeparator, SubsetKey destinationSubsetKey) {
        SortedMap<String, String> sortedMap = new TreeMap<String, String>();
        for (Motcle motcle : thesaurus.getMotcleList()) {
            if (CroisementUtils.isShouldNotCroisement(motcle, destinationSubsetKey)) {
                continue;
            }
            Label label = motcle.getLabels().getLangPartCheckedLabel(thesaurusLang);
            String idalpha = motcle.getIdalpha();
            if (label != null) {
                sortedMap.put(idalpha, label.getLabelString());
            } else {
                sortedMap.put(idalpha, "");
            }
        }
        StringBuilder buf = new StringBuilder();
        buf.append("[");
        boolean first = true;
        for (Map.Entry<String, String> entry : sortedMap.entrySet()) {
            if (first) {
                first = false;
            } else {
                buf.append(",");
            }
            String idalpha = entry.getKey();
            buf.append("{");
            buf.append("\"value\":\"");
            buf.append(idalpha);
            buf.append("\"");
            buf.append(",\"label\":\"");
            buf.append(idalpha);
            String label = entry.getValue();
            if (label.length() > 0) {
                buf.append(idalphaSeparator);
                escape(buf, label);
            }
            buf.append("\"");
            buf.append("}");
        }
        buf.append("]");
        return buf.toString();
    }

    private static void escape(StringBuilder buf, String s) {
        int length = s.length();
        for (int i = 0; i < length; i++) {
            char carac = s.charAt(i);
            switch (carac) {
                case '\"':
                    buf.append("\\\"");
                    break;
                case '\\':
                    buf.append("\\\\");
                    break;
                default:
                    buf.append(carac);
                    break;
            }
        }
    }

}
