/* FichothequeLib_API - Copyright (c) 2011-2023 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.fichotheque.utils;

import java.util.AbstractList;
import java.util.Collection;
import java.util.Collections;
import java.util.List;
import java.util.RandomAccess;
import net.fichotheque.FichothequeConstants;
import net.fichotheque.SubsetKey;
import net.fichotheque.corpus.metadata.FieldKey;
import net.fichotheque.externalsource.ExternalSourceDef;
import net.fichotheque.extraction.DataKey;
import net.fichotheque.extraction.def.AddendaExtractDef;
import net.fichotheque.extraction.def.AlbumExtractDef;
import net.fichotheque.extraction.def.CorpusExtractDef;
import net.fichotheque.extraction.def.FicheFilter;
import net.fichotheque.extraction.def.MotcleFilter;
import net.fichotheque.extraction.def.TagNameInfo;
import net.fichotheque.extraction.def.ThesaurusExtractDef;
import net.fichotheque.extraction.filterunit.AddendaExtractFilterUnit;
import net.fichotheque.extraction.filterunit.AlbumExtractFilterUnit;
import net.fichotheque.extraction.filterunit.ChronoFilterUnit;
import net.fichotheque.extraction.filterunit.CorpsdeficheFilterUnit;
import net.fichotheque.extraction.filterunit.CorpusExtractFilterUnit;
import net.fichotheque.extraction.filterunit.DataFilterUnit;
import net.fichotheque.extraction.filterunit.EnteteFilterUnit;
import net.fichotheque.extraction.filterunit.FieldKeyFilterUnit;
import net.fichotheque.extraction.filterunit.FieldNamePrefixFilterUnit;
import net.fichotheque.extraction.filterunit.FilterUnit;
import net.fichotheque.extraction.filterunit.LotFilterUnit;
import net.fichotheque.extraction.filterunit.MasterMotcleFilterUnit;
import net.fichotheque.extraction.filterunit.PhraseFilterUnit;
import net.fichotheque.extraction.filterunit.ThesaurusExtractFilterUnit;
import net.mapeadores.util.attr.AttributeUtils;
import net.mapeadores.util.attr.Attributes;
import net.fichotheque.extraction.filterunit.FicheParentageFilterUnit;


/**
 *
 * @author Vincent Calame
 */
public final class FilterUnitUtils {

    public final static EnteteFilterUnit ENTETE_FILTERUNIT = new InternalEnteteFilterUnit();
    public final static CorpsdeficheFilterUnit CORPSDEFICHE_FILTERUNIT = new InternalCorpsdeficheFilterUnit();
    public final static ChronoFilterUnit CHRONO_FILTERUNIT = new InternalChronoFilterUnit();
    public final static PhraseFilterUnit FICHEPHRASE_FILTERUNIT = new InternalPhraseFilterUnit(FichothequeConstants.FICHE_PHRASE);
    public final static List<FilterUnit> EMPTY_FILTERUNITLIST = Collections.emptyList();

    private FilterUnitUtils() {
    }

    public static FieldKeyFilterUnit toFieldKeyFilterUnit(FieldKey fieldKey, Attributes attributes) {
        return new InternalFieldKeyFilterUnit(fieldKey, attributes);
    }

    public static LotFilterUnit toLotFilterUnit(TagNameInfo tagNameInfo, List<FilterUnit> filterUnitList) {
        return new InternalLotFilterUnit(tagNameInfo, toImmutableList(filterUnitList));
    }

    public static FieldNamePrefixFilterUnit toFieldNamePrefixFilterUnit(short category, String prefix, Attributes attributes) {
        return new InternalFieldNamePrefixFilterUnit(category, prefix, attributes);
    }

    public static CorpusExtractFilterUnit toCorpusExtractFilterUnit(CorpusExtractDef corpusExtractDef, Attributes attributes) {
        return new InternalCorpusExtractFilterUnit(corpusExtractDef, attributes);
    }

    public static ThesaurusExtractFilterUnit toThesaurusExtractFilterUnit(ThesaurusExtractDef thesaurusExtractDef, Attributes attributes) {
        return new InternalThesaurusExtractFilterUnit(thesaurusExtractDef, attributes);
    }

    public static AlbumExtractFilterUnit toAlbumExtractFilterUnit(AlbumExtractDef albumExtractDef, Attributes attributes) {
        return new InternalAlbumExtractFilterUnit(albumExtractDef, attributes);
    }

    public static AddendaExtractFilterUnit toAddendaExtractFilterUnit(AddendaExtractDef addendaExtractDef, Attributes attributes) {
        return new InternalAddendaExtractFilterUnit(addendaExtractDef, attributes);
    }

    public static FicheParentageFilterUnit toFicheParentageFilterUnit(FicheFilter ficheFilter, Attributes attributes, Collection<SubsetKey> subsetKeys) {
        return new InternalFicheParentageFilterUnit(ficheFilter, attributes, FichothequeUtils.toList(subsetKeys));
    }

    public static MasterMotcleFilterUnit toMasterMotcleFilterUnit(MotcleFilter motcleFilter, Attributes attributes) {
        return new InternalMasterMotcleFilterUnit(motcleFilter, attributes);
    }

    public static PhraseFilterUnit toPhraseFilterUnit(String phraseName) {
        switch (phraseName) {
            case FichothequeConstants.FICHE_PHRASE:
                return FICHEPHRASE_FILTERUNIT;
            default:
                return new InternalPhraseFilterUnit(phraseName);
        }
    }

    public static DataFilterUnit toDataFilterUnit(DataKey dataKey, ExternalSourceDef externalSourceDef, Attributes attributes) {
        return new InternalDataFilterUnit(dataKey, externalSourceDef, attributes);
    }


    private static class InternalEnteteFilterUnit implements EnteteFilterUnit {

        private InternalEnteteFilterUnit() {
        }

        @Override
        public Attributes getAttributes() {
            return AttributeUtils.EMPTY_ATTRIBUTES;
        }

    }


    private static class InternalCorpsdeficheFilterUnit implements CorpsdeficheFilterUnit {

        private InternalCorpsdeficheFilterUnit() {
        }

        @Override
        public Attributes getAttributes() {
            return AttributeUtils.EMPTY_ATTRIBUTES;
        }

    }


    private static class InternalChronoFilterUnit implements ChronoFilterUnit {

        private InternalChronoFilterUnit() {
        }

        @Override
        public Attributes getAttributes() {
            return AttributeUtils.EMPTY_ATTRIBUTES;
        }

    }


    private static class InternalPhraseFilterUnit implements PhraseFilterUnit {

        private final String phraseName;

        private InternalPhraseFilterUnit(String phraseName) {
            this.phraseName = phraseName;
        }

        @Override
        public String getName() {
            return phraseName;
        }

        @Override
        public Attributes getAttributes() {
            return AttributeUtils.EMPTY_ATTRIBUTES;
        }

    }


    private static class InternalFieldKeyFilterUnit implements FieldKeyFilterUnit {

        private final FieldKey fieldKey;
        private final Attributes attributes;

        private InternalFieldKeyFilterUnit(FieldKey fieldKey, Attributes attributes) {
            this.fieldKey = fieldKey;
            this.attributes = attributes;
        }

        @Override
        public FieldKey getFieldKey() {
            return fieldKey;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }


    private static class InternalLotFilterUnit implements LotFilterUnit {

        private final TagNameInfo tagNameInfo;
        private final List<FilterUnit> filterUnitList;

        private InternalLotFilterUnit(TagNameInfo tagNameInfo, List<FilterUnit> filterUnitList) {
            this.tagNameInfo = tagNameInfo;
            this.filterUnitList = filterUnitList;
        }

        @Override
        public TagNameInfo getTagNameInfo() {
            return tagNameInfo;
        }

        @Override
        public List<FilterUnit> getFilterUnitList() {
            return filterUnitList;
        }

        @Override
        public Attributes getAttributes() {
            return AttributeUtils.EMPTY_ATTRIBUTES;
        }

    }


    private static class InternalFieldNamePrefixFilterUnit implements FieldNamePrefixFilterUnit {

        private final short category;
        private final String prefix;
        private final Attributes attributes;

        private InternalFieldNamePrefixFilterUnit(short category, String prefix, Attributes attributes) {
            this.category = category;
            this.prefix = prefix;
            this.attributes = attributes;
        }

        @Override
        public short getCategory() {
            return category;
        }

        @Override
        public String getPrefix() {
            return prefix;

        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }


    private static class InternalCorpusExtractFilterUnit implements CorpusExtractFilterUnit {

        private final CorpusExtractDef corpusExtractDef;
        private final Attributes attributes;

        private InternalCorpusExtractFilterUnit(CorpusExtractDef corpusExtractDef, Attributes attributes) {
            this.corpusExtractDef = corpusExtractDef;
            this.attributes = attributes;
        }

        @Override
        public CorpusExtractDef getCorpusExtractDef() {
            return corpusExtractDef;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }


    private static class InternalThesaurusExtractFilterUnit implements ThesaurusExtractFilterUnit {

        private final ThesaurusExtractDef thesaurusExtractDef;
        private final Attributes attributes;

        private InternalThesaurusExtractFilterUnit(ThesaurusExtractDef thesaurusExtractDef, Attributes attributes) {
            this.thesaurusExtractDef = thesaurusExtractDef;
            this.attributes = attributes;
        }

        @Override
        public ThesaurusExtractDef getThesaurusExtractDef() {
            return thesaurusExtractDef;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }


    private static class InternalAlbumExtractFilterUnit implements AlbumExtractFilterUnit {

        private final AlbumExtractDef albumExtractDef;
        private final Attributes attributes;

        private InternalAlbumExtractFilterUnit(AlbumExtractDef albumExtractDef, Attributes attributes) {
            this.albumExtractDef = albumExtractDef;
            this.attributes = attributes;
        }

        @Override
        public AlbumExtractDef getAlbumExtractDef() {
            return albumExtractDef;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }


    private static class InternalAddendaExtractFilterUnit implements AddendaExtractFilterUnit {

        private final AddendaExtractDef addendaExtractDef;
        private final Attributes attributes;

        private InternalAddendaExtractFilterUnit(AddendaExtractDef addendaExtractDef, Attributes attributes) {
            this.addendaExtractDef = addendaExtractDef;
            this.attributes = attributes;
        }

        @Override
        public AddendaExtractDef getAddendaExtractDef() {
            return addendaExtractDef;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }


    private static class InternalFicheParentageFilterUnit implements FicheParentageFilterUnit {

        private final FicheFilter ficheFilter;
        private final Attributes attributes;
        private final List<SubsetKey> subsetKeyList;

        private InternalFicheParentageFilterUnit(FicheFilter ficheFilter, Attributes attributes, List<SubsetKey> subsetKeyList) {
            this.ficheFilter = ficheFilter;
            this.attributes = attributes;
            this.subsetKeyList = subsetKeyList;
        }

        @Override
        public FicheFilter getFicheFilter() {
            return ficheFilter;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

        @Override
        public List<SubsetKey> getSubsetKeyList() {
            return subsetKeyList;
        }

    }


    private static class InternalMasterMotcleFilterUnit implements MasterMotcleFilterUnit {

        private final MotcleFilter motcleFilter;
        private final Attributes attributes;

        private InternalMasterMotcleFilterUnit(MotcleFilter motcleFilter, Attributes attributes) {
            this.motcleFilter = motcleFilter;
            this.attributes = attributes;
        }

        @Override
        public MotcleFilter getMotcleFilter() {
            return motcleFilter;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }


    private static class InternalDataFilterUnit implements DataFilterUnit {

        private final DataKey dataKey;
        private final ExternalSourceDef externalSourceDef;
        private final Attributes attributes;

        private InternalDataFilterUnit(DataKey dataKey, ExternalSourceDef externalSourceDef, Attributes attributes) {
            this.dataKey = dataKey;
            this.externalSourceDef = externalSourceDef;
            this.attributes = attributes;
        }

        @Override
        public DataKey getDataKey() {
            return dataKey;
        }

        @Override
        public ExternalSourceDef getExternalSourceDef() {
            return externalSourceDef;
        }

        @Override
        public Attributes getAttributes() {
            return attributes;
        }

    }

    public static List<FilterUnit> toImmutableList(Collection<FilterUnit> collection) {
        int size = collection.size();
        if (size == 0) {
            return EMPTY_FILTERUNITLIST;
        }
        return new FilterUnitList(collection.toArray(new FilterUnit[size]));
    }

    public static List<FilterUnit> wrap(FilterUnit[] array) {
        return new FilterUnitList(array);
    }


    private static class FilterUnitList extends AbstractList<FilterUnit> implements RandomAccess {

        private final FilterUnit[] array;

        private FilterUnitList(FilterUnit[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public FilterUnit get(int index) {
            return array[index];
        }

    }

}
